package cn.dankal.test.common.result;


import cn.dankal.test.common.entity.CommonConstant;
import cn.dankal.test.common.entity.CommonConstant.ResultCodeEnum;
import lombok.*;
import lombok.experimental.Accessors;

/**
 * {@link Getter}{@link Accessors}
 * 响应实体: Result
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/10/23
 */
@Getter
@Setter
@ToString
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
public class Result<T> {

    /**
     * 错误码
     */
    private String code;

    /**
     * 响应数据
     */
    private T data;

    /**
     * 响应消息
     */
    private String message;

    /**
     * 是否请求成功
     */
    private Boolean isSuccess;

    public Result(boolean isSuccess, T data, ResultCodeEnum resultCodeEnum) {
        this.isSuccess = isSuccess;
        this.data = data;
        this.code = resultCodeEnum.getCode();
        this.message = resultCodeEnum.getMessage();
    }


    /**
     * 成功响应结果
     *
     * @param data 响应数据
     * @return Result<T>
     */
    public static <T> Result<T> success(T data) {
        return new Result<>(true, data, ResultCodeEnum.SUCCESS);
    }

    /**
     * 失败响应结果
     *
     * @param message 失败提示信息
     * @return Result<String>
     */
    public static Result<String> error(String message) {
        CommonConstant.ResultCodeEnum resultCodeEnum2Fair = ResultCodeEnum.FAIL;
        resultCodeEnum2Fair.setMessage(message);
        return new Result<>(false, CommonConstant.CommonConst.MESSAGE_NO_DATA, resultCodeEnum2Fair);
    }

}
