package cn.dankal.test.order.controller;

import cn.dankal.test.common.entity.CommonConstant;
import cn.dankal.test.common.result.Result;
import cn.dankal.test.order.model.Order;
import cn.dankal.test.order.service.OrderService;
import io.seata.core.exception.TransactionException;
import org.springframework.web.bind.annotation.*;

/**
 * {@link RestController} {@link RequestMapping}
 * OrderApiController
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/10/22
 */
@RestController
@RequestMapping("/api/order")
public class OrderApiController {

    /**
     * orderService
     */
    private final OrderService orderService;

    public OrderApiController(OrderService orderService) {
        this.orderService = orderService;
    }

    /**
     * 下订单
     *
     * @param order order info
     * @return 操作结果(boolean)
     */
    @PostMapping("/place/order")
    public Result<Boolean> placeOrder(@RequestBody Order order) throws TransactionException {
        boolean placeOrder = orderService.placeOrder(order);
        String message = CommonConstant.ResultCodeEnum.SUCCESS.getMessage();
        String code = CommonConstant.ResultCodeEnum.SUCCESS.getCode();
        if (!placeOrder) {
            message = CommonConstant.ResultCodeEnum.FAIL.getMessage();
            code = CommonConstant.ResultCodeEnum.FAIL.getCode();
        }
        return new Result<Boolean>()
                .setIsSuccess(placeOrder)
                .setData(placeOrder)
                .setMessage(message)
                .setCode(code);
    }

    @GetMapping("/place/order/exception")
    public Result<Boolean> placeOrderException() throws TransactionException {

        // 无法正常回滚
        Boolean execResult = orderService.placeOrderException();

        //手动调用 API 回滚
        //Boolean execResult = orderService.placeOrderExceptionRollbackByApi();

        String message = CommonConstant.ResultCodeEnum.SUCCESS.getMessage();
        String code = CommonConstant.ResultCodeEnum.SUCCESS.getCode();
        if (!execResult) {
            message = CommonConstant.ResultCodeEnum.FAIL.getMessage();
            code = CommonConstant.ResultCodeEnum.FAIL.getCode();
        }
        return new Result<Boolean>()
                .setIsSuccess(execResult)
                .setData(execResult)
                .setMessage(message)
                .setCode(code);
    }
}
