package cn.dankal.test.storage.controller;

import cn.dankal.test.common.entity.CommonConstant;
import cn.dankal.test.common.result.Result;
import cn.dankal.test.storage.service.StorageService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * {@link RestController}{@link RequestMapping}
 * StorageApiController
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/10/22
 */
@RestController
@RequestMapping("/api/storage")
public class StorageApiController {

    /**
     * storageService
     */
    private final StorageService storageService;

    public StorageApiController(StorageService storageService) {
        this.storageService = storageService;
    }

    /**
     * 扣减库存
     *
     * @param commodityCode 商品编码
     * @param count         商品数量
     * @return 操作结果(boolean)
     */
    @GetMapping("/{commodityCode}/{count}")
    public Boolean deduct(
            @PathVariable("commodityCode") String commodityCode,
            @PathVariable("count") Integer count
    ) {
        return storageService.deduct(commodityCode, count);
    }

    /**
     * 扣减库存异常
     *
     * @return Result<Object>
     */
    @GetMapping("/deduct/exception")
    public Result<Object> deductWithException() {
        storageService.deductWithException();
        return new Result<>()
                .setMessage("throw an exception to test")
                .setCode(CommonConstant.ResultCodeEnum.FAIL.getCode())
                .setIsSuccess(false)
                .setData(false);
    }

}
