package cn.dankal.test.storage.service.impl;

import cn.dankal.test.storage.model.Storage;
import cn.dankal.test.storage.repository.StorageMapper;
import cn.dankal.test.storage.service.StorageService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * {@link StorageService} {@link ServiceImpl}
 * StorageServiceImpl
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/10/22
 */
@Slf4j
@Service
public class StorageServiceImpl extends ServiceImpl<StorageMapper, Storage> implements StorageService {

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deduct(String commodityCode, int count) throws IllegalArgumentException {
        LambdaQueryWrapper<Storage> storageQueryWrapper = new LambdaQueryWrapper<Storage>()
                .eq(Storage::getCommodityCode, commodityCode);
        Storage storage = this.getOne(storageQueryWrapper);
        if (null == storage) {
            // 商品不存在
            throw new IllegalArgumentException("商品不存在: " + commodityCode);
        }
        Long realCount = storage.getCount();
        if (realCount < count) {
            if (realCount==0){
                log.warn("商品 {} 库存不足: {}", commodityCode, realCount);
            }
            return false;
        }
        // 扣减库存
        storage.setCount(realCount - count);
        this.updateById(storage);
        return true;
    }

    @Override
    public Boolean deductWithException() {
        throw new RuntimeException("测试分布式事务提交异常");
    }
}
