package cn.dankal.test.client;

import cn.dankal.test.client.fallback.StorageServiceFallback;
import cn.dankal.test.common.result.Result;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

/**
 * {@link FeignClient}
 * StorageServiceClient: storage-service
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2020/10/22
 * @see StorageServiceFallback
 */
@FeignClient(value = "storage-service", fallback = StorageServiceFallback.class)
public interface StorageServiceClient {

    /**
     * 扣减库存
     *
     * @param commodityCode 商品编码
     * @param count         商品数量
     * @return 操作结果(boolean)
     */
    @GetMapping("/api/storage/{commodityCode}/{count}")
    Boolean deduct(
            @PathVariable("commodityCode") String commodityCode,
            @PathVariable("count") Integer count
    );

    /**
     * 测试 事务提交异常
     *
     * @return 如果操作成功则返回 true,否则 返回 false
     */
    @GetMapping("/api/storage/deduct/exception")
    Result<Object> deductWithException();
}
